package com.example.fuzzControll.controller;

import com.example.fuzzControll.exception.CmdException;
import com.example.fuzzControll.exception.FileException;
import com.example.fuzzControll.pojo.vo.AjaxResult;
import com.example.fuzzControll.service.SeedFileService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.List;

/**
 * aflnet种子文件
 */
@Slf4j
@RestController
@RequestMapping("/seedFile")
public class SeedFileController {

    @Autowired
    SeedFileService service;

    /**
     * 种子文件查询接口
     */
    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public AjaxResult list() {
        List<String> files = null;
        try {
            files = service.getSeedFiles();
        } catch (CmdException e) {
            log.error(e.getDefaultMessage());
            return AjaxResult.error();
        }
        return AjaxResult.success(files);
    }

    /**
     * 种子文件删除接口
     */
    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    public AjaxResult delete(@RequestParam("fileName") String fileName) {
        try {
            service.delFile(fileName);
        } catch (CmdException e) {
            log.error(e.getDefaultMessage());
            return AjaxResult.error("种子文件删除失败！");
        }
        return AjaxResult.success("种子文件删除成功！");
    }


    /**
     * seeFileUpload
     */
    @RequestMapping(value = "/upload", method = RequestMethod.POST)
    public AjaxResult upload(@RequestParam("file") MultipartFile file) {
        try {
            service.upload(file);
        } catch (FileException | CmdException e) {
            log.error(e.getDefaultMessage());
            return AjaxResult.error("上传文件失败！");
        }
        return AjaxResult.success("上传文件成功！");
    }
}
