package com.example.fuzzControll.service.impl;

import com.example.fuzzControll.exception.MysqlException;
import com.example.fuzzControll.mapper.AflnetMapper;
import com.example.fuzzControll.mapper.KittyMapper;
import com.example.fuzzControll.mapper.MissionInfoMapper;
import com.example.fuzzControll.pojo.vo.*;
import com.example.fuzzControll.service.FuzzLogService;
import com.example.fuzzControll.tools.FileTools;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service("FuzzLogService")
public class FuzzLogServiceImpl implements FuzzLogService {
    @Autowired
    MissionInfoMapper missionInfoMapper;
    @Autowired
    AflnetMapper aflnetMapper;
    @Autowired
    KittyMapper kittyMapper;
    FileTools fileTools = new FileTools();

    @Override
    public List<FuzzLogTransEntity> getFuzzLog(int missionId) throws IllegalAccessException {
        MissionInfo missionInfo = missionInfoMapper.selectByMissionId(missionId);
        if (missionInfo == null) {
            return null;
        }
        switch (missionInfo.getTableId()) {
            case 1:
                return downloadAflnetFile(missionInfo);
            case 2:
                return downloadKittyProtocalFile(missionInfo);
            case 3:
                return downloadKittyOtherMethodFile(missionInfo);
            default:
                throw new IllegalAccessException("Invalid mission!");
        }
    }

    @Transactional(rollbackFor = MysqlException.class)
    public List<FuzzLogTransEntity> downloadKittyProtocalFile(MissionInfo missionInfo) {
        /*获取结果日志*/
        List<FuzzLogTransEntity> fuzzLogList = new ArrayList<FuzzLogTransEntity>();
        KittyResult kittyResult = kittyMapper.getKittyResultByMissionId(missionInfo.getMissionId());
        StringBuffer buffer = new StringBuffer();
        buffer.append(kittyResult.getResultError()).append(System.lineSeparator()).append(kittyResult.getResultOut());
        FuzzLogTransEntity fuzzLogTransEntity = new FuzzLogTransEntity(buffer.toString().getBytes(), missionInfo.getMissionName() + ".txt");
        fuzzLogList.add(fuzzLogTransEntity);
        /*获取数据包文件*/
//        KittyPackageFile kittyPackageFile = kittyMapper.getKittyPackageFileById(missionInfo.getMissionId());
//        FuzzLogTransEntity fuzzLogTransEntitySend = new FuzzLogTransEntity(kittyPackageFile.getKittySendFile(), kittyPackageFile.getKittyRecvFileName());
//        FuzzLogTransEntity fuzzLogTransEntityRecv = new FuzzLogTransEntity(kittyPackageFile.getKittyRecvFile(), kittyPackageFile.getKittyRecvFileName());
//        fuzzLogList.add(fuzzLogTransEntitySend);
//        fuzzLogList.add(fuzzLogTransEntityRecv);
        return fuzzLogList;
    }

    @Transactional(rollbackFor = MysqlException.class)
    public List<FuzzLogTransEntity> downloadKittyOtherMethodFile(MissionInfo missionInfo) {
        List<FuzzLogTransEntity> fuzzLogList = new ArrayList<FuzzLogTransEntity>();
        KittyResult kittyResult = kittyMapper.getKittyResultByMissionId(missionInfo.getMissionId());
        StringBuffer buffer = new StringBuffer();
        buffer.append(kittyResult.getResultError()).append(System.lineSeparator()).append(kittyResult.getResultOut());
        FuzzLogTransEntity fuzzLogTransEntity = new FuzzLogTransEntity(buffer.toString().getBytes(), missionInfo.getMissionName() + ".txt");
        fuzzLogList.add(fuzzLogTransEntity);
        return fuzzLogList;
    }

    @Transactional(rollbackFor = MysqlException.class)
    public List<FuzzLogTransEntity> downloadAflnetFile(MissionInfo missionInfo) {
        List<FuzzLogTransEntity> fuzzLogList = new ArrayList<FuzzLogTransEntity>();
        AflnetResult aflnetResult = aflnetMapper.selectResultById(missionInfo.getMissionId());
        FuzzLogTransEntity fuzzLogTransEntity = new FuzzLogTransEntity(aflnetResult.getFile(), aflnetResult.getFileName());
        fuzzLogList.add(fuzzLogTransEntity);
        return fuzzLogList;
    }
}
